@@ -1,5 +1,5 @@
 /* diskio.c --- Read and write data structures from disk.
- * Copyright (C) 2002, 2003, 2004  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2006  Simon Josefsson
  *
  * This file is part of Shishi.
  *
@@ -101,6 +101,33 @@ _shishi_save_data (Shishi * handle, FILE * fh, Shishi_asn1 asn1,
   return SHISHI_OK;
 }
 
+int
+shishi_padata_print (Shishi * handle, FILE * fh, Shishi_asn1 padata)
+{
+  return _shishi_print_armored_data (handle, fh, padata, "PA-DATA", NULL);
+}
+
+int
+shishi_methoddata_print (Shishi * handle, FILE * fh, Shishi_asn1 methoddata)
+{
+  return _shishi_print_armored_data (handle, fh, methoddata,
+				     "METHOD-DATA", NULL);
+}
+
+int
+shishi_etype_info_print (Shishi * handle, FILE * fh, Shishi_asn1 etypeinfo)
+{
+  return _shishi_print_armored_data (handle, fh, etypeinfo,
+				     "ETYPE-INFO", NULL);
+}
+
+int
+shishi_etype_info2_print (Shishi * handle, FILE * fh, Shishi_asn1 etypeinfo2)
+{
+  return _shishi_print_armored_data (handle, fh, etypeinfo2,
+				     "ETYPE-INFO2", NULL);
+}
+
 int
 shishi_enckdcreppart_print (Shishi * handle,
 			    FILE * fh, Shishi_asn1 enckdcreppart)
@@ -819,7 +846,7 @@ shishi_key_parse (Shishi * handle, FILE * fh, Shishi_key ** key)
   char armorend[BUFSIZ];
   int in_key = 0, in_body = 0;
   int res;
-  size_t len, buflen;
+  size_t len;
   Shishi_key *lkey = NULL;
 
   sprintf (armorbegin, HEADERBEG, "KEY");
