@@ -34,6 +34,8 @@
 #include <string.h>
 #endif
 
+#include <stdbool.h>
+
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -118,6 +120,7 @@
 #include "xvasprintf.h"
 #include "base64.h"
 #include "getdate.h"
+#include "getline.h"
 #include "strtok_r.h"
 #include "time_r.h"
 #include "timegm.h"
