@@ -27,7 +27,7 @@
 #include <time.h>		/* time_t */
 #include <shishi-int.h>		/* uint32_t */
 
-#define SHISHI_VERSION "0.0.23"
+#define SHISHI_VERSION "0.0.24"
 
 /* Error codes */
 typedef enum
@@ -560,6 +560,11 @@ struct Shishi_tkts_hint
   time_t starttime;
   time_t endtime;
   time_t renew_till;
+  int32_t preauthetype;
+  char *preauthsalt;
+  size_t preauthsaltlen;
+  char *preauths2kparams;
+  size_t preauths2kparamslen;
 };
 typedef struct Shishi_tkts_hint Shishi_tkts_hint;
 
@@ -877,6 +882,10 @@ extern int shishi_ticket_parse (Shishi * handle, FILE * fh,
 				Shishi_asn1 * ticket);
 extern int shishi_ticket_read (Shishi * handle, FILE * fh,
 			       Shishi_asn1 * ticket);
+extern int shishi_etype_info_print (Shishi * handle, FILE * fh,
+				    Shishi_asn1 etypeinfo);
+extern int shishi_etype_info2_print (Shishi * handle, FILE * fh,
+				     Shishi_asn1 etypeinfo2);
 
 /* authenticator.c */
 extern Shishi_asn1 shishi_authenticator (Shishi * handle);
@@ -1141,6 +1150,9 @@ extern int shishi_kdcreq_add_padata (Shishi * handle,
 extern int shishi_kdcreq_add_padata_tgs (Shishi * handle,
 					 Shishi_asn1 kdcreq,
 					 Shishi_asn1 apreq);
+extern int shishi_kdcreq_add_padata_preauth (Shishi * handle,
+					     Shishi_asn1 kdcreq,
+					     Shishi_key *key);
 extern int shishi_kdcreq_build (Shishi * handle, Shishi_asn1 kdcreq);
 
 /* kdc.c */
@@ -1367,6 +1379,9 @@ extern const char *shishi_krberror_errorcode_message (Shishi * handle,
 						      int errorcode);
 extern const char *shishi_krberror_message (Shishi * handle,
 					    Shishi_asn1 krberror);
+extern int shishi_krberror_methoddata (Shishi * handle,
+				       Shishi_asn1 krberror,
+				       Shishi_asn1 *methoddata);
 
 /* gztime.c */
 extern const char *shishi_generalize_time (Shishi * handle, time_t t);
@@ -1592,6 +1607,12 @@ extern int shishi_asn1_write_bitstring (Shishi * handle, Shishi_asn1 node,
 
 extern void shishi_asn1_done (Shishi * handle, Shishi_asn1 node);
 
+extern Shishi_asn1 shishi_asn1_pa_enc_ts_enc (Shishi * handle);
+extern Shishi_asn1 shishi_asn1_encrypteddata (Shishi * handle);
+extern Shishi_asn1 shishi_asn1_padata (Shishi * handle);
+extern Shishi_asn1 shishi_asn1_methoddata (Shishi * handle);
+extern Shishi_asn1 shishi_asn1_etype_info (Shishi * handle);
+extern Shishi_asn1 shishi_asn1_etype_info2 (Shishi * handle);
 extern Shishi_asn1 shishi_asn1_asreq (Shishi * handle);
 extern Shishi_asn1 shishi_asn1_asrep (Shishi * handle);
 extern Shishi_asn1 shishi_asn1_tgsreq (Shishi * handle);
@@ -1620,6 +1641,16 @@ extern Shishi_msgtype shishi_der_msgtype (Shishi * handle,
 
 extern Shishi_asn1 shishi_der2asn1 (Shishi * handle,
 				    const char *der, size_t derlen);
+extern Shishi_asn1 shishi_der2asn1_padata (Shishi * handle,
+					   const char *der, size_t derlen);
+extern Shishi_asn1 shishi_der2asn1_methoddata (Shishi * handle,
+					       const char *der, size_t derlen);
+extern Shishi_asn1 shishi_der2asn1_etype_info (Shishi * handle,
+					       const char *der,
+					       size_t derlen);
+extern Shishi_asn1 shishi_der2asn1_etype_info2 (Shishi * handle,
+						const char *der,
+						size_t derlen);
 extern Shishi_asn1 shishi_der2asn1_ticket (Shishi * handle,
 					   const char *der, size_t derlen);
 extern Shishi_asn1 shishi_der2asn1_encticketpart (Shishi * handle,
