@@ -1,5 +1,5 @@
 /* tkts.c --- Ticket set handling.
- * Copyright (C) 2002, 2003, 2004  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2006  Simon Josefsson
  *
  * This file is part of Shishi.
  *
@@ -407,7 +407,7 @@ shishi_tkts_write (Shishi_tkts * tkts, FILE * fh)
       if (res != SHISHI_OK)
 	{
 	  shishi_error_printf (tkts->handle,
-			       "Could not print ticket: %s\n",
+			       "Could not print ticket: %s",
 			       shishi_error (tkts->handle));
 	  return res;
 	}
@@ -417,7 +417,7 @@ shishi_tkts_write (Shishi_tkts * tkts, FILE * fh)
       if (res != SHISHI_OK)
 	{
 	  shishi_error_printf (tkts->handle,
-			       "Could not print ticket: %s\n",
+			       "Could not print ticket: %s",
 			       shishi_error (tkts->handle));
 	  return res;
 	}
@@ -427,7 +427,7 @@ shishi_tkts_write (Shishi_tkts * tkts, FILE * fh)
       if (res != SHISHI_OK)
 	{
 	  shishi_error_printf (tkts->handle,
-			       "Could not print ticket: %s\n",
+			       "Could not print ticket: %s",
 			       shishi_error (tkts->handle));
 	  return res;
 	}
@@ -871,6 +871,280 @@ set_tgtflags_based_on_hint (Shishi_tkts_hint * tkthint,
 
   if (tkthint->endtime)
     tgthint->endtime = tkthint->endtime;
+
+  if (tkthint->passwd)
+    tgthint->passwd = tkthint->passwd;
+
+  if (tkthint->preauthetype)
+    tgthint->preauthetype = tkthint->preauthetype;
+
+  if (tkthint->preauthsalt)
+    {
+      tgthint->preauthsalt = tkthint->preauthsalt;
+      tgthint->preauthsaltlen = tkthint->preauthsaltlen;
+    }
+
+  if (tkthint->preauths2kparams)
+    {
+      tgthint->preauths2kparams = tkthint->preauths2kparams;
+      tgthint->preauths2kparamslen = tkthint->preauths2kparamslen;
+    }
+}
+
+/* Pre-authenticate request, based on LOCHINT.  Currently only
+   PA-ENC-TIMESTAMP is supported.  */
+static int
+do_preauth (Shishi_tkts *tkts, Shishi_tkts_hint *lochint, Shishi_as *as)
+{
+  int rc = SHISHI_OK;
+
+  if (lochint->preauthetype)
+    {
+      Shishi_key *key;
+      char *user;
+
+      /* XXX Don't prompt for password here? */
+
+      rc = shishi_asreq_clientrealm (tkts->handle, shishi_as_req (as),
+				     &user, NULL);
+      if (rc != SHISHI_OK)
+	return rc;
+
+      rc = shishi_prompt_password (tkts->handle, &lochint->passwd,
+				   "Enter password for `%s': ", user);
+      if (rc != SHISHI_OK)
+	return rc;
+
+      if (!lochint->preauthsalt)
+	{
+	  rc = shishi_derive_default_salt (tkts->handle, user,
+					   &lochint->preauthsalt);
+	  if (rc != SHISHI_OK)
+	    return rc;
+
+	  lochint->preauthsaltlen = strlen (lochint->preauthsalt);
+	}
+
+      rc = shishi_key_from_string (tkts->handle, lochint->preauthetype,
+				   lochint->passwd, strlen (lochint->passwd),
+				   lochint->preauthsalt,
+				   lochint->preauthsaltlen,
+				   lochint->preauths2kparams, &key);
+      if (rc != SHISHI_OK)
+	return rc;
+
+      rc = shishi_kdcreq_add_padata_preauth (tkts->handle,
+					     shishi_as_req (as), key);
+    }
+
+  return rc;
+}
+
+/* Handle ETYPE-INFO and ETYPE-INFO2 pre-auth data. */
+static int
+recover_preauth_info (Shishi_tkts *tkts,
+		      Shishi_as *as,
+		      Shishi_tkts_hint *lochint,
+		      Shishi_asn1 einfos,
+		      bool isinfo2,
+		      bool *retry)
+{
+  size_t foundpos = SIZE_MAX;
+  size_t i, n;
+  int rc;
+
+  if (VERBOSENOISE(tkts->handle))
+    printf ("Found INFO-ETYPE(2) pre-auth hints, trying to find etype...\n");
+
+  if (VERBOSEASN1(tkts->handle))
+    {
+      if (isinfo2)
+	shishi_etype_info2_print (tkts->handle, stdout, einfos);
+      else
+	shishi_etype_info_print (tkts->handle, stdout, einfos);
+    }
+
+  if (lochint->preauthetype)
+    {
+      if (VERBOSENOISE(tkts->handle))
+	printf ("Pre-auth data already specified...\n");
+      return SHISHI_OK;
+    }
+
+  rc = shishi_asn1_number_of_elements (tkts->handle, einfos, "", &n);
+  if (rc != SHISHI_OK)
+    return rc;
+
+  for (i = 1; i <= n; i++)
+    {
+      char *format;
+      int32_t etype;
+
+      format = xasprintf ("?%d.etype", i);
+      rc = shishi_asn1_read_int32 (tkts->handle, einfos, format, &etype);
+      free (format);
+      if (rc == SHISHI_OK)
+	{
+	  size_t j;
+
+	  if (VERBOSENOISE(tkts->handle))
+	    printf ("Server has etype %d...\n", etype);
+
+	  for (j = 0; j < tkts->handle->nclientkdcetypes; j++)
+	    {
+	      if (etype == tkts->handle->clientkdcetypes[j])
+		{
+		  if (j < foundpos && VERBOSENOISE(tkts->handle))
+		    {
+		      char *salt;
+		      size_t saltlen;
+
+		      printf ("New best etype %d...\n", etype);
+
+		      /* XXX mem leak. */
+
+		      format = xasprintf ("?%d.salt", i);
+		      rc = shishi_asn1_read (tkts->handle, einfos, format,
+					     &lochint->preauthsalt,
+					     &lochint->preauthsaltlen);
+		      free (format);
+		      if (rc != SHISHI_OK && rc != SHISHI_ASN1_NO_ELEMENT)
+			return rc;
+
+		      if (isinfo2)
+			{
+			  format = xasprintf ("?%d.s2kparams", i);
+			  rc = shishi_asn1_read (tkts->handle, einfos, format,
+						 &lochint->preauths2kparams,
+						 &lochint->preauths2kparamslen);
+			  free (format);
+			  if (rc != SHISHI_OK && rc != SHISHI_ASN1_NO_ELEMENT)
+			    return rc;
+			}
+		    }
+
+		  foundpos = MIN(foundpos, j);
+		}
+	    }
+	}
+    }
+
+  if (foundpos != SIZE_MAX)
+    {
+      lochint->preauthetype = tkts->handle->clientkdcetypes[foundpos];
+
+      if (VERBOSENOISE(tkts->handle))
+	printf ("Best pre-auth etype was %d...\n", lochint->preauthetype);
+
+      *retry = true;
+    }
+
+  return SHISHI_OK;
+}
+
+/* Called when KDC refused with a NEED_PREAUTH error.  This function
+   should look at the METHOD-DATA, figure out what kind of pre-auth is
+   requested, and if it is able to figure out how to recover from the
+   error, set *RETRY to true and set any hints in LOCHINT that help
+   do_preauth() compute the proper pre-auth data.  */
+static int
+recover_preauth (Shishi_tkts *tkts,
+		 Shishi_as *as,
+		 Shishi_tkts_hint *lochint,
+		 bool *retry)
+{
+  Shishi_asn1 krberror = shishi_as_krberror (as);
+  Shishi_asn1 pas;
+  size_t i, n;
+  int rc;
+
+  *retry = false;
+
+  if (VERBOSE(tkts->handle))
+    printf ("Server requested pre-auth data, figuring out what to send...\n");
+
+  rc = shishi_krberror_methoddata (tkts->handle, krberror, &pas);
+  if (rc != SHISHI_OK)
+    return rc;
+
+  rc = shishi_asn1_number_of_elements (tkts->handle, pas, "", &n);
+  if (rc == SHISHI_OK)
+    {
+      for (i = 1; i <= n; i++)
+	{
+	  char *format = xasprintf ("?%d.padata-type", i);
+	  int32_t padatatype;
+
+	  rc = shishi_asn1_read_int32 (tkts->handle, pas, format,
+					&padatatype);
+	  free (format);
+	  if (rc == SHISHI_OK)
+	    {
+	      if (VERBOSENOISE(tkts->handle))
+		printf ("Looking at pa-type %d...\n", padatatype);
+
+	      switch (padatatype)
+		{
+		  /* XXX Don't parse INFO structures if there is a
+		     INFO2. */
+
+		case SHISHI_PA_ETYPE_INFO:
+		case SHISHI_PA_ETYPE_INFO2:
+		  {
+		    char *der;
+		    size_t len;
+		    Shishi_asn1 einfos;
+
+		    format = xasprintf ("?%d.padata-value", i);
+		    rc = shishi_asn1_read (tkts->handle, pas, format,
+					   &der, &len);
+		    free (format);
+		    if (rc != SHISHI_OK)
+		      {
+			shishi_error_printf (tkts->handle,
+					     "Can't extract PA-DATA value");
+			continue;
+		      }
+
+		    if (padatatype == SHISHI_PA_ETYPE_INFO)
+		      einfos = shishi_der2asn1_etype_info (tkts->handle,
+							   der, len);
+		    else
+		      einfos = shishi_der2asn1_etype_info2 (tkts->handle,
+							    der, len);
+		    free (der);
+		    if (!einfos)
+		      {
+			shishi_error_printf (tkts->handle,
+					     "Can't DER decode PA-DATA");
+			continue;
+		      }
+
+		    rc = recover_preauth_info
+		      (tkts, as, lochint, einfos,
+		       padatatype == SHISHI_PA_ETYPE_INFO2, retry);
+		    if (rc != SHISHI_OK)
+		      {
+			shishi_error_printf (tkts->handle,
+					     "Could not use pre-auth data: %s",
+					     shishi_strerror (rc));
+			continue;
+		      }
+
+		    shishi_asn1_done (tkts->handle, einfos);
+		  }
+		  break;
+
+		default:
+		  break;
+		}
+	    }
+	}
+    }
+
+  shishi_asn1_done (tkts->handle, pas);
+
+  return SHISHI_OK;
 }
 
 /**
@@ -914,26 +1188,48 @@ shishi_tkts_get_tgt (Shishi_tkts * tkts, Shishi_tkts_hint * hint)
   if (tgt)
     return tgt;
 
+ again:
   rc = shishi_as (tkts->handle, &as);
   if (rc == SHISHI_OK)
     rc = act_hint_on_kdcreq (tkts->handle, &lochint, shishi_as_req (as));
+  if (rc == SHISHI_OK)
+    rc = do_preauth (tkts, &lochint, as);
   if (rc == SHISHI_OK)
     rc = shishi_as_req_build (as);
   if (rc == SHISHI_OK)
     rc = shishi_as_sendrecv_hint (as, &lochint);
   if (rc == SHISHI_OK)
-    rc = shishi_as_rep_process (as, NULL, hint->passwd);
+    rc = shishi_as_rep_process (as, NULL, lochint.passwd);
+  if (rc == SHISHI_GOT_KRBERROR
+      && shishi_krberror_errorcode_fast (tkts->handle,
+					 shishi_as_krberror (as))
+      == SHISHI_KDC_ERR_PREAUTH_REQUIRED)
+    {
+      bool retry = false;
+
+      rc = recover_preauth (tkts, as, &lochint, &retry);
+      if (rc != SHISHI_OK)
+	return NULL;
+
+      if (retry)
+	{
+	  shishi_as_done (as);
+	  goto again;
+	}
+
+      shishi_error_printf (tkts->handle, "Unsupported pre-auth required");
+      return NULL;
+    }
   if (rc != SHISHI_OK)
     {
       shishi_error_printf (tkts->handle,
 			   "AS exchange failed: %s\n%s\n",
 			   shishi_strerror (rc), shishi_error (tkts->handle));
-      if (rc == SHISHI_GOT_KRBERROR)
-	shishi_krberror_pretty_print (tkts->handle, stdout,
-				      shishi_as_krberror (as));
       return NULL;
     }
 
+  /* XXX free lochint members */
+
   tgt = shishi_as_tkt (as);
   if (!tgt)
     {
